package errorlink;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.debug.ui.console.FileLink;
import org.eclipse.debug.ui.console.IConsole;
import org.eclipse.debug.ui.console.IConsoleLineTracker;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IRegion;

/**
 * Generates hyperlinks for compiler output
 */
public class RegexpLineTracker implements IConsoleLineTracker
{

	/* 
	 * Jikes error format is filename:lineNum:colNum:otherLineNum:otherColNum: Error: "message"
	 */
	public static final String JIKES_ERROR_REGEXP =
		"\\s*\\[javac\\]\\s*(.*\\.java):(\\d+):(\\d+):\\d+:\\d+:";

	static
	{
		// Do this here so that the defaults are setup the first time we use this class - no lifecycle due to being a fragment
		PreferencePage.setPreferenceStoreDefaults();
	}

	private IConsole fConsole;
	private List regexpPatterns;

	/**
	 * Constructor for JavacLineTracker.
	 */
	public RegexpLineTracker()
	{
		super();
	}

	/**
	 * @see org.eclipse.debug.ui.console.IConsoleLineTracker#init(org.eclipse.debug.ui.console.IConsole)
	 */
	public void init(IConsole console)
	{
		fConsole = console;
		regexpPatterns = new ArrayList();
		String[] regexps = PreferencePage.getRegexps();
		for (int i = 0; i < regexps.length; i++)
		{
			String regexp = regexps[i];
			regexpPatterns.add(Pattern.compile(regexp));
		}
	}

	public void lineAppended(IRegion line)
	{
		try
		{
			int lineOffset = line.getOffset();
			int lineLength = line.getLength();
			String text = fConsole.getDocument().get(lineOffset, lineLength);
			for (Iterator iter = regexpPatterns.iterator(); iter.hasNext();)
			{
				Pattern pattern = (Pattern) iter.next();
				doMatch(pattern, text, lineOffset, lineLength);
			}
		}
		catch (BadLocationException e)
		{
		}
	}

	/**
	 * @see org.eclipse.debug.ui.console.IConsoleLineTracker#lineAppended(org.eclipse.jface.text.IRegion)
	 */
	public void doMatch(
		Pattern pattern,
		String text,
		int lineOffset,
		int lineLength)
	{
		Matcher matcher = pattern.matcher(text);
		if (matcher.lookingAt())
		{
			IFile file = null;
			String fileName = null;
			int lineNum = -1;
			int colNum = -1;
			int linkOffset = 0;
			int linkLength = 0;

			try
			{
				fileName = matcher.group(1);
				lineNum = Integer.parseInt(matcher.group(2));
				colNum = Integer.parseInt(matcher.group(3));
				// hyperlink starts at start of first group
				linkOffset = lineOffset + matcher.start(1);
				// hyperlink extends up to end of third group, depending on number of groups in regexp
				linkLength = matcher.end(1) - matcher.start(1);
				linkLength = matcher.end(2) - matcher.start(1);
				linkLength = matcher.end(3) - matcher.start(1);
			}
			catch (IndexOutOfBoundsException e)
			{
			}
			catch (NumberFormatException e)
			{
			}

			if (fileName != null)
			{
				file =
					ResourcesPlugin
						.getWorkspace()
						.getRoot()
						.getFileForLocation(
						new Path(fileName));
			}
			if (file != null && file.exists())
			{
				FileLink link = new FileLink(file, null, -1, -1, lineNum);
				fConsole.addLink(link, linkOffset, linkLength);
			}
		}
	}

	/**
	 * @see org.eclipse.debug.ui.console.IConsoleLineTracker#dispose()
	 */
	public void dispose()
	{
		fConsole = null;
	}

}
